@@ -39,12 +39,17 @@
 
 #include "dixstruct.h"
 #include "extnsionst.h"
+#include "randrstr.h"
 #include <X11/X.h>
 #include <X11/extensions/panoramiXproto.h>
 
 #include "vmware.h"
 #include "vmwarectrlproto.h"
 
+#ifndef HAVE_XORG_SERVER_1_5_0
+#include <xf86_ansic.h>
+#include <xf86_libc.h>
+#endif
 
 /*
  *----------------------------------------------------------------------------
@@ -122,7 +127,7 @@ VMwareCtrlDoSetRes(ScrnInfoPtr pScrn,
       VmwareLog(("DoSetRes: %d %d\n", x, y));
 
       if (resetXinerama) {
-         xfree(pVMWARE->xineramaNextState);
+         free(pVMWARE->xineramaNextState);
          pVMWARE->xineramaNextState = NULL;
          pVMWARE->xineramaNextNumOutputs = 0;
       }
@@ -276,7 +281,7 @@ VMwareCtrlDoSetTopology(ScrnInfoPtr pScrn,
 
       VmwareLog(("DoSetTopology: %d %d\n", maxX, maxY));
 
-      xineramaState = (VMWAREXineramaPtr)xcalloc(number, sizeof(VMWAREXineramaRec));
+      xineramaState = (VMWAREXineramaPtr)calloc(number, sizeof(VMWAREXineramaRec));
       if (xineramaState) {
          memcpy(xineramaState, extents, number * sizeof (VMWAREXineramaRec));
 
@@ -289,43 +294,27 @@ VMwareCtrlDoSetTopology(ScrnInfoPtr pScrn,
           * will be no mode change. In this case, push it out
           * immediately.
           */
-         xfree(pVMWARE->xineramaNextState);
+         free(pVMWARE->xineramaNextState);
          pVMWARE->xineramaNextState = xineramaState;
          pVMWARE->xineramaNextNumOutputs = number;
 
          if (maxX == pVMWARE->ModeReg.svga_reg_width &&
              maxY == pVMWARE->ModeReg.svga_reg_height) {
 
-            /*
-             * XXX:
-             *
-             * There are problems with trying to set a Xinerama state
-             * without a mode switch. The biggest one is that
-             * applications typically won't notice a topology change
-             * that occurs without a mode switch. If you run "xdpyinfo
-             * -ext XINERAMA" after one such topology change, it will
-             * report the new data, but apps (like the GNOME Panel)
-             * will not notice until the next mode change.
-             *
-             * I don't think there's any good solution to this... as
-             * far as I know, even on a non-virtualized machine
-             * there's no way for an app to find out if the Xinerama
-             * opology changes without a resolution change also
-             * occurring. There might be some cheats we can take, like
-             * swithcing to a new mode with the same resolution and a
-             * different (fake) refresh rate, or temporarily switching
-             * to an intermediate mode. Ick.
-             *
-             * The other annoyance here is that when we reprogram the
-             * SVGA device's monitor topology registers, it may
-             * rearrange those monitors on the host's screen, but they
-             * will still have the old contents. This might be
-             * correct, but it isn't guaranteed to match what's on X's
-             * framebuffer at the moment. So we'll send a
-             * full-framebuffer update rect afterwards.
-             */
+	    /*
+	     * The annoyance here is that when we reprogram the
+	     * SVGA device's monitor topology registers, it may
+	     * rearrange those monitors on the host's screen, but they
+	     * will still have the old contents. This might be
+	     * correct, but it isn't guaranteed to match what's on X's
+	     * framebuffer at the moment. So we'll send a
+	     * full-framebuffer update rect afterwards.
+	     */
 
             vmwareNextXineramaState(pVMWARE);
+#ifdef HAVE_XORG_SERVER_1_2_0
+            RRSendConfigNotify(pScrn->pScreen);
+#endif
             vmwareSendSVGACmdUpdateFullScreen(pVMWARE);
 
             return TRUE;
